package com.hand.adf.widgets.weather.view.backing;

import com.hand.adf.widgets.weather.view.backing.city.GoogleCityHandler;
import com.hand.adf.widgets.weather.view.backing.weather.GoogleWeatherHandler;
import com.hand.adf.widgets.weather.view.backing.weather.WeatherSet;

import java.net.SocketTimeoutException;

import java.net.URL;

import java.net.URLConnection;

import java.net.UnknownHostException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;

import javax.faces.event.ValueChangeEvent;

import javax.faces.model.SelectItem;

import javax.servlet.http.Cookie;

import javax.servlet.http.HttpServletRequest;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import oracle.adf.view.rich.component.rich.input.RichSelectOneChoice;
import oracle.adf.view.rich.component.rich.layout.RichPanelGroupLayout;
import oracle.adf.view.rich.context.AdfFacesContext;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

public class WeatherWidget {
    private RichPanelGroupLayout weatherPanel;
    private Map weatherCity;
    private List<Map> weatherForecast;
    private Map currentWeather;
    private Map cityList;

    public WeatherWidget() {

        // get city from session context
        weatherCity = this.getWeatherCityBySession();

        if (weatherCity == null ||
            !weatherCity.getClass().equals(HashMap.class)) { // get city from cookie context
            weatherCity = this.getWeatherCityByCookie();
        }

        if (weatherCity == null ||
            !weatherCity.getClass().equals(HashMap.class)) { // get default city from xml
            this.weatherCity = this.getWeatherCityByDefault();
        }

        if (weatherCity == null) { // default value is beijing
            weatherCity = new HashMap();
            weatherCity.put("name", "北京");
            weatherCity.put("latitude_e6", "39930000");
            weatherCity.put("longitude_e6", "116279998");
        }

    }

    public Map getWeatherCityByDefault() {
        Map result = null;
        for (Object cityKey : this.getCityList().keySet()) {
            if (((Map)this.getCityList().get(cityKey)).get("default") !=
                null &&
                ((Map)this.getCityList().get(cityKey)).get("default").equals(true)) {
                result = (Map)this.getCityList().get(cityKey);
                break;
            }
        }
        return result;
    }

    public Map getWeatherCityByCookie() {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        String cityNameInCookie = null;
        Map result = null;

        Cookie[] cookie =
            ((HttpServletRequest)facesContext.getExternalContext().getRequest()).getCookies();
        try {
            if (cookie != null) {
                for (Cookie curr : cookie) {
                    if (curr.getName().equals("com.hand.adf.widgets.weather.WeatherWidget")) {
                        cityNameInCookie = curr.getValue();
                        String[] cityNameCharList =
                            cityNameInCookie.split("%u");
                        StringBuffer sb = new StringBuffer();
                        if (cityNameCharList != null) {
                            for (String currChar : cityNameCharList) {
                                if (!currChar.equals("")) {
                                    sb.append((char)Integer.parseInt(currChar,
                                                                     16));
                                }
                            }
                        }
                        cityNameInCookie = sb.toString();
                        break;
                    }
                }
            }
        } catch (Exception e) {
            cityNameInCookie = null;
        }
        if (cityNameInCookie != null) { // from cookie
            result = (Map)this.getCityList().get(cityNameInCookie);
        }

        return result;
    }

    public Map getWeatherCityBySession() {
        FacesContext facesContext = FacesContext.getCurrentInstance();

        Map weatherCityInSession =
            (Map)facesContext.getExternalContext().getSessionMap().get("com.hand.adf.widgets.weather.WeatherWidget");

        return weatherCityInSession;
    }

    /**
     * Query weather forecast conditions by Google API
     */
    public void queryWeatherForecast() {

        ArrayList<Map> weather = new ArrayList<Map>();
        Map currentWeather = new HashMap();

        URL url;

        try {

            url =
 new URL("http://www.google.com/ig/api?hl=zh-cn&weather=,,," + this.weatherCity.get("latitude_e6") +
         "," + this.weatherCity.get("longitude_e6"));

            SAXParserFactory spf = SAXParserFactory.newInstance();
            SAXParser sp = spf.newSAXParser();
            XMLReader xr = sp.getXMLReader();
            WeatherSet ws = new WeatherSet();
            GoogleWeatherHandler gwh = new GoogleWeatherHandler(ws);
            xr.setContentHandler(gwh);
            URLConnection weatherConn = url.openConnection();
            weatherConn.setReadTimeout(5000); // set 5 second timeout
            InputSource is = new InputSource(weatherConn.getInputStream());
            is.setEncoding("GBK");
            xr.parse(is);

            if (ws.getForecastConditions().size() == 0) {
                weatherNetExecptionHandler(weather, currentWeather);
            } else {
                weather = ws.getForecastConditions();
                currentWeather = ws.getCurrentConditionsMap();
            }

        } catch (SocketTimeoutException e) {
            weatherNetExecptionHandler(weather, currentWeather);
        } catch (UnknownHostException e) {
            weatherNetExecptionHandler(weather, currentWeather);
        } catch (Exception e) {
            weatherNetExecptionHandler(weather, currentWeather);
        }

        this.weatherForecast = weather;
        this.currentWeather = currentWeather;
    }

    public List<SelectItem> getCityListAsSelectItem() {
        Map cityList = this.getCityList();
        List<SelectItem> cityListAsSelectItem = new ArrayList<SelectItem>();
        Iterator cityIter = cityList.values().iterator();
        while (cityIter.hasNext()) {
            Map city = (Map)cityIter.next();
            cityListAsSelectItem.add(new SelectItem(city.get("name")));
        }
        return cityListAsSelectItem;
    }


    /**
     * Query city list by Google API
     */
    public void queryCityList() {

        Map cityList = new HashMap();

        URL url;
        try {
            url =
 new URL("http://www.google.com/ig/cities?&output=xml&hl=zh-cn&country=cn");

            SAXParserFactory spf = SAXParserFactory.newInstance();
            SAXParser sp = spf.newSAXParser();
            XMLReader xr = sp.getXMLReader();
            GoogleCityHandler gch = new GoogleCityHandler(cityList);
            xr.setContentHandler(gch);
            URLConnection cityConn = url.openConnection();
            cityConn.setReadTimeout(5000); // set 5 second timeout
            InputSource is = new InputSource(cityConn.getInputStream());
            is.setEncoding("GBK");
            xr.parse(is);

        } catch (SocketTimeoutException e) {
            cityNetExecptionHandler(cityList);
        } catch (UnknownHostException e) {
            cityNetExecptionHandler(cityList);
        } catch (Exception e) {
            cityNetExecptionHandler(cityList);
        }

        this.cityList = cityList;
    }

    public void weatherCityChange(String targetCity) {
        this.weatherCity = (Map)this.getCityList().get(targetCity);
        if (this.weatherCity == null) {
            this.queryCityList();
            weatherCityChange(targetCity);
        }
        // refresh weather forecast panel
        AdfFacesContext afc = AdfFacesContext.getCurrentInstance();
        afc.addPartialTarget(this.weatherPanel);

        // add current city to session
        FacesContext facesContext = FacesContext.getCurrentInstance();
        facesContext.getExternalContext().getSessionMap().put("com.hand.adf.widgets.weather.WeatherWidget",
                                                              this.weatherCity);
    }

    private void weatherNetExecptionHandler(List<Map> targetList,
                                            Map currentWeather) {
        HashMap tempMap = new HashMap();
        tempMap.put("day_of_week", "");
        tempMap.put("low", "");
        tempMap.put("high", "");
        tempMap.put("icon", "");
        tempMap.put("condition", "服务暂不可用");
        targetList.add((Map)tempMap.clone());
        targetList.add((Map)tempMap.clone());
        targetList.add((Map)tempMap.clone());

        currentWeather.put("condition", "");
        currentWeather.put("temp_f", "");
        currentWeather.put("temp_c", "");
        currentWeather.put("humidity", "");
        currentWeather.put("wind_condition", "");
        currentWeather.put("icon", "");
    }

    private void cityNetExecptionHandler(Map targetMap) {
        HashMap tempMap = new HashMap();
        tempMap.put("name", "服务暂不可用");
        tempMap.put("latitude_e6", "");
        tempMap.put("longitude_e6", "");
        targetMap.put(tempMap.get("name"), tempMap);
    }

    public void weatherCityChange(ValueChangeEvent valueChangeEvent) {
        RichSelectOneChoice choice =
            (RichSelectOneChoice)valueChangeEvent.getSource();
        this.weatherCityChange((String)choice.getValue());
    }

    public void setWeatherPanel(RichPanelGroupLayout weatherPanel) {
        this.weatherPanel = weatherPanel;
    }

    public RichPanelGroupLayout getWeatherPanel() {
        return weatherPanel;
    }

    public Map getWeatherCity() {
        return weatherCity;
    }

    public List<Map> getWeatherForecast() {
        if (weatherForecast == null) {
            this.queryWeatherForecast();
        }
        return weatherForecast;
    }

    public Map getCityList() {
        if (this.cityList == null) {
            this.queryCityList();
        }
        return this.cityList;
    }

    public Map getCurrentWeather() {
        if (currentWeather == null) {
            this.queryWeatherForecast();
        }
        return currentWeather;
    }
}
